
/**************************************************************************
 *
 *  $Id: mbgddmsg.h 1.5 2006/06/19 15:26:19 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Print or remove debug messages by redefinitions.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgddmsg.h $
 *  Revision 1.5  2006/06/19 15:26:19  martin
 *  Fixed compiler warnings if DEBUG or DBG not defined.
 *  Revision 1.4  2002/06/12 12:25:54  martin
 *  Bug fix: check for target MBG_TGT_WIN32 instead of MBG_TGT_W32.
 *  Revision 1.3  2002/02/19 14:50:48Z  MARTIN
 *  Added support for Win32.
 *  Revision 1.2  2002/02/19 09:28:00  MARTIN
 *  Use new header mbg_tgt.h to check the target environment.
 *  Revision 1.1  2001/03/02 13:51:23  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _MBGDDMSG_H
#define _MBGDDMSG_H


#if defined( DEBUG ) || ( defined( DBG ) && DBG )

#include <mbg_tgt.h>

enum
{
  MBG_DBG_WARN,
  MBG_DBG_INFO,
  MBG_DBG_DETAIL,
  MBG_DBG_INIT_DEV,
  MBG_DBG_INIT_PCI,
  MBG_DBG_INIT_MCA,
  MBG_DBG_INIT_ISA,
  MBG_DBG_INIT_RSRC,
  MBG_DBG_IRQ,
  N_MBG_DBG_LVL
};

extern int debug;

#define _chk_lvl( _lvl )  ( (_lvl) < debug )

#if defined( MBG_TGT_NETWARE )
  #include <conio.h>
  #define _printf ConsolePrintf
  #define _hd
  #define _tl "\n"
#elif defined( MBG_TGT_OS2 )
  #include <iprintf.h>
  #define _printf iprintf
  #define _hd
  #define _tl "\n"
#elif defined( MBG_TGT_WIN32 )
  #include <ntddk.h>
  #define _printf DbgPrint
  #define _hd
  #define _tl "\n"
#elif defined( MBG_TGT_LINUX )
  // #include <printk.h>
  #define _printf printk
  #define _hd KERN_INFO
  #define _tl "\n"
#else  // MBG_TGT_QNX, MBG_TGT_DOS, ...
  #include <stdio.h>
  #define _printf printf
  #define _hd
  #define _tl "\n"
#endif


#define _mbgddmsg_0( _lvl, _fmt ) \
{                                 \
  if ( _chk_lvl( _lvl ) )         \
    { _printf( _hd _fmt _tl ); }  \
}

#define _mbgddmsg_1( _lvl, _fmt, _p1 )  \
{                                       \
  if ( _chk_lvl( _lvl ) )               \
    { _printf( _hd _fmt _tl, (_p1) ); } \
}

#define _mbgddmsg_2( _lvl, _fmt, _p1, _p2 )    \
{                                              \
  if ( _chk_lvl( _lvl ) )                      \
    { _printf( _hd _fmt _tl, (_p1), (_p2) ); } \
}

#define _mbgddmsg_3( _lvl, _fmt, _p1, _p2, _p3 )      \
{                                                     \
  if ( _chk_lvl( _lvl ) )                             \
    { _printf( _hd _fmt _tl, (_p1), (_p2), (_p3) ); } \
}

#define _mbgddmsg_4( _lvl, _fmt, _p1, _p2, _p3, _p4 )        \
{                                                            \
  if ( _chk_lvl( _lvl ) )                                    \
    { _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4) ); } \
}

#define _mbgddmsg_5( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5 )          \
{                                                                   \
  if ( _chk_lvl( _lvl ) )                                           \
    { _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4), (_p5) ); } \
}

#define _mbgddmsg_6( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6 )            \
{                                                                          \
  if ( _chk_lvl( _lvl ) )                                                  \
    { _printf( _hd _fmt _tl, (_p1), (_p2), (_p3), (_p4), (_p5), (_p6) ); } \
}

#else

  #define _mbgddmsg_0( _lvl, _fmt );
  #define _mbgddmsg_1( _lvl, _fmt, _p1 );
  #define _mbgddmsg_2( _lvl, _fmt, _p1, _p2 );
  #define _mbgddmsg_3( _lvl, _fmt, _p1, _p2, _p3 );
  #define _mbgddmsg_4( _lvl, _fmt, _p1, _p2, _p3, _p4 );
  #define _mbgddmsg_5( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5 );
  #define _mbgddmsg_6( _lvl, _fmt, _p1, _p2, _p3, _p4, _p5, _p6 );

#endif

#endif  /* _MBGDDMSG_H */
