
/**************************************************************************
 *
 *  $Id: parmgps.c 1.3 2004/11/09 14:21:36 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Functions to handle/convert Meinberg GPS specific parameters.
 *
 * -----------------------------------------------------------------------
 *  $Log: parmgps.c $
 *  Revision 1.3  2004/11/09 14:21:36  martin
 *  Redefined some data types using C99 fixed-size definitions.
 *  Revision 1.2  2002/02/19 13:30:23Z  MARTIN
 *  Bug fix in port_settings_from_port_parm_mode().
 *  Revision 1.1  2002/01/30 10:30:26  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#define _PARMGPS
 #include <parmgps.h>
#undef _PARMGPS

#include <myutil.h>
#include <string.h>



/*HDR*/
int get_str_idx( const char *search,
                 const char *str_table[],
                 int n_entries )
{
  int i;

  for ( i = 0; i < n_entries; i++ )
    if ( strcmp( search, str_table[i] ) == 0 )
      return i;

  return -1;

}  // get_str_idx



/*HDR*/
int get_baud_rate_idx( BAUD_RATE baud_rate )
{
  int i;

  for ( i = 0; i < N_MBG_BAUD_RATES; i++ )
    if ( baud_rate == mbg_baud_rate[i] )
      return i;

  return -1;

}  // get_baud_rate_idx



/*HDR*/
int get_framing_idx( const char *framing )
{
  return get_str_idx( framing, mbg_framing_str, N_MBG_FRAMINGS );

}  // get_framing_idx



/*HDR*/
void port_settings_from_port_parm_mode(
       PORT_SETTINGS *p_ps,
       uint8_t pp_mode,
       int str_type_cap
     )
{
  if ( pp_mode >= STR_UCAP )
  {
    p_ps->str_type = str_type_cap;
    p_ps->mode = ( pp_mode == STR_UCAP ) ? STR_AUTO : STR_ON_REQ;
  }
  else
  {
    p_ps->str_type = 0;
    p_ps->mode = pp_mode;
  }

}  // port_settings_from_port_parm_mode



/*HDR*/
void port_parm_mode_from_port_settings(
       uint8_t *pp_mode,
       const PORT_SETTINGS *p_ps,
       int str_type_cap
     )
{
  if ( p_ps->str_type == str_type_cap )
    *pp_mode = ( p_ps->mode == STR_ON_REQ ) ? STR_UCAP_REQ : STR_UCAP;
  else
    *pp_mode = p_ps->mode;

}  // port_parm_mode_from_port_settings



/*HDR*/
void port_settings_from_port_parm(
  PORT_SETTINGS *p_ps,
  int port_num,
  const PORT_PARM *p_pp,
  int cap_str_idx
)
{
  p_ps->parm = p_pp->com[port_num];

  port_settings_from_port_parm_mode( p_ps, p_pp->mode[port_num],
                                     cap_str_idx );

}  // port_info_from_port_parm



/*HDR*/
void port_parm_from_port_settings(
  PORT_PARM *p_pp,
  int port_num,
  const PORT_SETTINGS *p_ps,
  int cap_str_idx
)
{
  p_pp->com[port_num] = p_ps->parm;

  port_parm_mode_from_port_settings( &p_pp->mode[port_num],
                                     p_ps, cap_str_idx );

}  // port_parm_from_port_settings



/*HDR*/
int valid_port_info( const PORT_INFO *p,
                     const STR_TYPE_INFO_IDX str_type_info_idx[],
                     int n_str_type )
{
  const PORT_SETTINGS *p_ps = &p->port_settings;
  ulong supp_modes;
  int idx;


  idx = get_baud_rate_idx( p_ps->parm.baud_rate );
  if ( !_inrange( idx, 0, N_MBG_BAUD_RATES ) )
    goto fail;

  if ( !_is_supported( idx, p->supp_baud_rates ) )
    goto fail;


  idx = get_framing_idx( p_ps->parm.framing );
  if ( !_inrange( idx, 0, N_MBG_FRAMINGS ) )
    goto fail;

  if ( !_is_supported( idx, p->supp_framings ) )
    goto fail;


  if ( p_ps->parm.handshake != HS_NONE )
    goto fail;


  idx = p_ps->str_type;

  if ( idx >= n_str_type )           /* str_type is always >= 0 */
    goto fail;

  if ( !_is_supported( idx, p->supp_str_types ) )
    goto fail;

  // Use the str_type index to get the supported output mode mask
  // from the string type inf table. This will be needed to
  // check whether the current mode is supported by the selected
  // string type.
  supp_modes = str_type_info_idx[idx].str_type_info.supp_modes;

  idx = p_ps->mode;

  if ( idx >= N_STR_MODE )           /* mode is always >= 0 */
    goto fail;

  if ( !_is_supported( idx, supp_modes ) )
    goto fail;

  if ( p_ps->flags > 0 )             /* currently always 0 */
    goto fail;


  if ( p->supp_baud_rates & ~_mask( N_MBG_BAUD_RATES ) )
    goto fail;

  return 1;


fail:
  return 0;

}  // valid_port_info


