
/**************************************************************************
 *
 *  $Id: pci_asic.h 1.8 2006/06/14 12:59:12 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions for the Meinberg PCI interface ASIC.
 *
 * -----------------------------------------------------------------------
 *  $Log: pci_asic.h $
 *  Revision 1.8  2006/06/14 12:59:12  martin
 *  Added support for TCR511PCI.
 *  Revision 1.7  2006/03/10 10:47:03  martin
 *  Added support for PCI511.
 *  Revision 1.6  2005/11/03 15:30:44Z  martin
 *  Added support for GPS170PCI.
 *  Revision 1.5  2004/11/09 12:51:56Z  martin
 *  Redefined fixed width data types using standard C99 types.
 *  Defined some constants unsigned.
 *  Revision 1.4  2004/10/14 15:01:23  martin
 *  Added support for TCR167PCI.
 *  Revision 1.3  2003/05/13 14:38:55Z  MARTIN
 *  Added ushort fields to unions PCI_ASIC_REG and 
 *  PCI_ASIC_ADDON_DATA.
 *  Revision 1.2  2003/04/03 10:56:38  martin
 *  Use unions for registers.
 *  Modified BADR0 initializer due to fixed size of address decoder.
 *  Revision 1.1  2003/02/07 11:42:52  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _PCI_ASIC_H
#define _PCI_ASIC_H


/* Other headers to be included */

#include <words.h>
#include <use_pack.h>

#if defined( _USE_PACK )      // set byte alignment
  #pragma pack( 1 )
#endif


#ifdef _PCI_ASIC
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */


typedef struct
{
  uint32_t cfg_class_rev_id;
  uint16_t cfg_badr_0;
  uint16_t cfg_dev_id;
} PCI_ASIC_CFG;


typedef union
{
  uint32_t ul;
  uint16_t us[2];
  uint8_t b[4];
} PCI_ASIC_REG;


typedef union
{
  uint32_t ul[4];
  uint16_t us[8];
  uint8_t b[16];
} PCI_ASIC_ADDON_DATA;


typedef struct
{
  PCI_ASIC_CFG cfg;             // writeable from add-on once after power-up
  PCI_ASIC_REG features[2];     // readonly, currently 0

  PCI_ASIC_REG status_port;
  PCI_ASIC_REG control_status;     // codes defined below
  PCI_ASIC_REG pci_data;           // pass byte from PCI bus to add-on
  PCI_ASIC_REG reserved_1;

  PCI_ASIC_ADDON_DATA addon_data;  // returns data from add-on to PCI bus
  PCI_ASIC_ADDON_DATA reserved_2;  // currently not implemented
} PCI_ASIC;


// The following bits are used with the control_status register.
// All other bits are reserved for future use.

// The IRQ flag for the add-on side is set whenever data is
// written to the cmd register. It is cleared if the add-on
// microcontroller writes this bit back to the control_status
// register. If the bit is set, the add-on signals /ADD_ON_IRQ
// and ADD_ON_BUSY are asserted.
#define PCI_ASIC_ADD_ON_IRQF   0x00000001UL

// The IRQ flag for the PCI bus is set whenever the add-on
// microcontroller asserts the ASIC's /PCI_IRQ line, or the
// add-on microcontroller sets this bit to 1. It is cleared
// if this bit is written back from the PCI side. If the bit
// is set, an IRQ is asserted on the PCI bus.
#define PCI_ASIC_PCI_IRQF      0x00010000UL


// The ASIC's address decoder always decodes 8 bits, so
// each device must request at least that number of
// addresses from the PCI BIOS:
#define PCI_ASIC_ADDR_RANGE    0x100U


// Initializers for device configurations

#define PCPS_DEV_CLASS_CODE   0x08800000UL
#define PCI_ASIC_BADR0_INIT   ( ~( PCI_ASIC_ADDR_RANGE - 1 ) | 0x01 )


#define PCI_ASIC_CFG_PCI510          \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_PCI510 )         \
}

#define PCI_ASIC_CFG_GPS169PCI       \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_GPS169PCI )      \
}

#define PCI_ASIC_CFG_TCR510PCI       \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_TCR510PCI )      \
}

#define PCI_ASIC_CFG_TCR167PCI       \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_TCR167PCI )      \
}

#define PCI_ASIC_CFG_GPS170PCI       \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_GPS170PCI )      \
}

#define PCI_ASIC_CFG_PCI511          \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_PCI511 )         \
}

#define PCI_ASIC_CFG_TCR511PCI       \
{                                    \
  _hilo_32( PCPS_DEV_CLASS_CODE ),   \
  _hilo_16( PCI_ASIC_BADR0_INIT ),   \
  _hilo_16( PCI_DEV_TCR511PCI )      \
}

/* function prototypes: */

#ifdef __cplusplus
extern "C" {
#endif

/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

/* (no header definitions found) */

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USE_PACK )      // set default alignment
  #pragma pack()
#endif

/* End of header body */

#undef _ext
#undef _DO_INIT

#endif  /* _PCI_ASIC_H */

