
/**************************************************************************
 *
 *  $Id: pcpsdrvr.h 1.26 2006/07/07 09:44:23 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for pcpsdrvr.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: pcpsdrvr.h $
 *  Revision 1.26  2006/07/07 09:44:23  martin
 *  Fixed definition of control macros for the case where _PCPS_USE_PCIMGR is overridden
 *  from the command line.
 *  Revision 1.25  2006/06/19 15:31:09  martin
 *  Added support for TCR511PCI.
 *  Updated function prototypes.
 *  Revision 1.24  2006/03/10 11:01:51  martin
 *  Added support for PCI511.
 *  Revision 1.23  2005/11/03 15:50:45Z  martin
 *  Added support for GPS170PCI.
 *  Revision 1.22  2005/06/02 10:35:09Z  martin
 *  Added macro _pcps_ddev_is_pci_amcc().
 *  Added macro _pcps_ddev_has_generic_io().
 *  Updated function prototypes.
 *  Revision 1.21  2004/12/09 11:03:38Z  martin
 *  Support configuration of on-board frequency synthesizer.
 *  Revision 1.20  2004/11/09 13:05:12Z  martin
 *  Fixed syntax bug in macro _pcps_ddev_fw_rev_num().
 *  New macro _pcps_ddev_has_gps_data().
 *  New macro _pcps_ddev_requires_irig_workaround().
 *  Revision 1.19  2004/10/14 15:01:24Z  martin
 *  Added support for TCR167PCI.
 *  Revision 1.18  2004/09/06 15:11:04Z  martin
 *  Support a GPS_DATA interface where sizes are specified 
 *  by 16 instead of the original 8 bit quantities, thus allowing 
 *  to transfer data blocks which exceed 255 bytes.
 *  Revision 1.17  2004/04/14 10:29:45Z  martin
 *  Pack structures 1 byte aligned.
 *  Revision 1.16  2004/04/07 09:47:19Z  martin
 *  New macros _pcps_ddev_has_irig() and
 *  _pcps_ddev_has_irig_tx().
 *  Revision 1.15  2004/03/10 17:32:23Z  martin
 *  Use CLOCKS_PER_SEC for timeout under QNX6 (Neutrino).
 *  Revision 1.14  2003/11/17 16:15:01  martin
 *  Support clock tick timeout for QNX.
 *  Revision 1.13  2003/07/08 15:07:32Z  martin
 *  Simplified definitions of default preprocessor macros.
 *  Compile for plug'n'play for Linux kernels 2.4.0 or newer.
 *  Updated function prototypes.
 *  Revision 1.12  2003/06/19 09:56:29  MARTIN
 *  Renamed macro ..clr_cap_buffer to ..clr_ucap_buffer.
 *  New macro _pcps_ddev_has_ucap().
 *  Changes due to renamed symbols.
 *  Updated function prototypes.
 *  Revision 1.11  2003/05/16 09:31:54  MARTIN
 *  Increased timeout loop count from 0x1000 to 0x7FFFFF.
 *  Rearranged inclusion of headers depending on the target.
 *  Added array for ISA port addresses.
 *  Revision 1.10  2003/04/09 16:30:24  martin
 *  Supports PCI510, GPS169PCI, and TCR510PCI,
 *  and new PCI_ASIC used by those devices.
 *  Renamed macro _pcps_ddev_is_irig() to _pcps_ddev_is_irig_rx().
 *  New macros _pcps_ddev_has_ref_offs(), _pcps_ddev_has_opt_flags().
 *  Preliminary support for PCPS_TZDL.
 *  Revision 1.9  2002/08/09 08:53:53  MARTIN
 *  New macro _pcps_ddev_can_clr_cap_buff().
 *  New macro _pcps_ddev_is_irig().
 *  New macro _pcps_ddev_has_signal().
 *  New macro _pcps_ddev_has_mod().
 *  Revision 1.8  2002/02/26 09:34:03  MARTIN
 *  Removed macro _pcps_read_sernum() which was replaced
 *  by a function pcps_read_sernum() which reads the S/N from 
 *  any clock that supports a S/N.
 *  Updated function prototypes.
 *  Revision 1.7  2002/02/19 09:28:01  MARTIN
 *  Use new header mbg_tgt.h to check the target environment.
 *  Revision 1.6  2002/02/01 12:00:10  MARTIN
 *  Added new definitions for GPS168PCI.
 *  Renamed macro _pcps_ddev_rev_num to _pcps_ddev_fw_rev_num
 *  to follow naming conventions.
 *  Source code cleanup.
 *  Revision 1.5  2001/11/30 09:52:48  martin
 *  Added support for event_time which, however, requires 
 *  a custom GPS firmware.
 *  Revision 1.4  2001/10/16 10:15:44  MARTIN
 *  New Macro _pcps_ddev_has_serial_hs() which determines 
 *  whether DCF77 clock supports baud rate higher than default.
 *  Added some macros and comments  corresponding to 
 *  pcpsdev.h.
 *  Revision 1.3  2001/09/18 06:53:57  MARTIN
 *  Two sets of preprocessor symbols for Win9x/ME and WinNT/2k.
 *  New preprocessor symbol controls usage of clock ticks for timeout.
 *  Changed type of PCPS_RSRC.irq_num from int to ushort.
 *  Updated function prototypes.
 *  Revision 1.2  2001/03/16 14:45:34  MARTIN
 *  New functions and definitions to support PNP drivers.
 *  Revision 1.1  2001/03/01 16:29:22  MARTIN
 *  Initial version for the new library.
 *
 **************************************************************************/

#ifndef _PCPSDRVR_H
#define _PCPSDRVR_H

// Setup default controls to include support for
// special features.

#include <mbg_tgt.h>

#if defined( MBG_TGT_NETWARE )
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR      0
  #define _DEFAULT_PCPS_USE_RSRCMGR     0
  #define _DEFAULT_PCPS_USE_MCA         0
#elif defined( MBG_TGT_OS2 )
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  0
  #define _DEFAULT_PCPS_USE_PCIMGR      0
  #define _DEFAULT_PCPS_USE_RSRCMGR     1
  #define _DEFAULT_PCPS_USE_MCA         1
#elif defined( MBG_TGT_WIN32_PNP )
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR      1
  #define _DEFAULT_PCPS_USE_RSRCMGR     0
  #define _DEFAULT_PCPS_USE_MCA         0
#elif defined( MBG_TGT_WIN32 )
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR      0
  #define _DEFAULT_PCPS_USE_RSRCMGR     0
  #define _DEFAULT_PCPS_USE_MCA         0
#elif defined( MBG_TGT_LINUX )
  #include <linux/version.h>
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR \
    ( defined( KERNEL_VERSION ) && \
    ( LINUX_VERSION_CODE >= KERNEL_VERSION( 2, 4, 0 ) ) )
  #define _DEFAULT_PCPS_USE_RSRCMGR     1
  #define _DEFAULT_PCPS_USE_MCA         0
#elif defined( MBG_TGT_QNX )
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR      0
  #define _DEFAULT_PCPS_USE_RSRCMGR     0
  #define _DEFAULT_PCPS_USE_MCA         0
#else  // DOS ...
  #define _DEFAULT_PCPS_USE_CLOCK_TICK  1
  #define _DEFAULT_PCPS_USE_PCIMGR      0
  #define _DEFAULT_PCPS_USE_RSRCMGR     0
  #define _DEFAULT_PCPS_USE_MCA         1
#endif

#ifndef _PCPS_USE_CLOCK_TICK
  #define _PCPS_USE_CLOCK_TICK  _DEFAULT_PCPS_USE_CLOCK_TICK
#endif

#ifndef _PCPS_USE_PCIMGR
  #define _PCPS_USE_PCIMGR      _DEFAULT_PCPS_USE_PCIMGR
#endif

#ifndef _PCPS_USE_PCIBIOS
  #define _PCPS_USE_PCIBIOS     !_PCPS_USE_PCIMGR
#endif

#ifndef _PCPS_USE_RSRCMGR
  #define _PCPS_USE_RSRCMGR     _DEFAULT_PCPS_USE_RSRCMGR
#endif

#ifndef _PCPS_USE_MCA
  #define _PCPS_USE_MCA         _DEFAULT_PCPS_USE_MCA
#endif

#define _PCPS_USE_PCI           ( _PCPS_USE_PCIMGR || _PCPS_USE_PCIBIOS )




/* Other headers to be included */

#include <pcpsdev.h>
#include <use_pack.h>

#include <stddef.h>

#if defined( MBG_TGT_DOS )
  #include <string.h>
  #include <time.h>
  #include <conio.h>
  #include <xportio.h>
#endif

#if defined( MBG_TGT_WIN32 )
  #include <mbg_w32.h>
#endif

#if defined( MBG_TGT_LINUX )
  #include <mbg_lx.h>

  #ifdef __KERNEL__   // preliminary ...
    #include <linux/proc_fs.h>
  #else
    #include <linux/fs.h>
  #endif
#endif

#if defined( MBG_TGT_QNX )
  #include <mbg_qnx.h>
  #include <string.h>
  #include <time.h>
#endif

#if defined( MBG_TGT_NETWARE )
  #include <string.h>
  #include <time.h>
  #include <conio.h>
#endif

#if defined( MBG_TGT_OS2 )
  #ifndef OS2_INCLUDED
    #define INCL_DOSSEMAPHORES
    #include <os2.h>
    #include <rmbase.h>
  #endif

  #include <string.h>
  #include <time.h>
  #include <conio.h>
  #include <xportio.h>
#else
  #define FAR
#endif

#if _PCPS_USE_RSRCMGR
  #include <rsrc.h>
#endif

#ifdef _PCPSDRVR
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )   // set byte alignment
  #pragma pack( 1 )
#endif


/* ------ definitions used with PCI clocks -------------------------- */

/* Default timeout count accessing the board */

#if _PCPS_USE_CLOCK_TICK
  #if defined( MBG_TGT_NETWARE )
    #define PCPS_TIMEOUT_CNT ( (ulong)( 200 * __get_CLK_TCK() ) / 1000 )
  #elif defined( MBG_TGT_LINUX )
    #define PCPS_TIMEOUT_CNT ( (ulong)( 200 * HZ ) / 1000 )
  #elif defined( MBG_TGT_WIN32 )
    #define PCPS_TIMEOUT_CNT ( (ulong)( 200 * MBG_TICKS_PER_SEC ) / 1000 )
  #elif defined( MBG_TGT_QNX_NTO )
    #define PCPS_TIMEOUT_CNT ( (ulong)( 200 * CLOCKS_PER_SEC ) / 1000 )
  #else
    #define PCPS_TIMEOUT_CNT ( (ulong)( 200 * CLK_TCK ) / 1000 )
  #endif
#else
  #define PCPS_TIMEOUT_CNT   0x7FFFFFUL
#endif


typedef struct
{
  // the device info data
  PCPS_DEV dev;

  // the read function to be used to access the clock
  short (*read)( PCPS_DEV_CFG *pdev, uchar cmd,
                 uchar FAR *buffer, uchar count );

  #if _PCPS_USE_RSRCMGR
    #if defined( MBG_TGT_OS2 )
      PCPS_HDEV hDev;
      RSRC_LIST rsrc;
    #endif
  #endif

} PCPS_DDEV;



// The structure below is used to pass resource information
// from a PNP manager to the device initializer function.

typedef struct
{
  PCPS_PORT_RSRC port[N_PCPS_PORT_RSRC];
  ushort irq_num;
} PCPS_RSRC;



/* The PCI vendor ID and device ID numbers are used to detect a
 * PCI clock in a system and query which resources have been
 * assigned by the BIOS.
 * (PCI vendor ID and PCI device IDs are defined in PCPSDEFS.H)
 */

/* the number of address lines decoded by a PCI clock */
#define PCPS_DECODE_WIDTH_PCI   16


/* ------ definitions used with MCA clocks -------------------------- */

/* The MCA adapter ID number is used to detect a MCA clock in a
 * system and query which resources have been assigned by the
 * system's POS (programmable option select).
 */

/* MCA Adapter ID numbers */
#define MCA_ID_PS31        0x6AAC   /* assigned by IBM */
#define MCA_ID_PS31_OLD    0x6303   /* assigned by Meinberg, used with */
                                    /* the first series of PS31 boards */

/* the total number of ports acquired by a MCA clock */
#define PCPS_NUM_PORTS_MCA      16

/* the number of address lines decoded by a MCA clock */
#define PCPS_DECODE_WIDTH_MCA   16


/* ------ definitions used with ISA clocks -------------------------- */

/* A board ID for the newer clocks with ISA bus. The number can
 * be read at port_base+2 (low byte) and port_base+3 (high byte)
 * of ISA clocks. This ID number matches the MCA adapter ID
 * defined above and is not available on PC31 clocks.
 */
#define ISA_ID_PCPS             MCA_ID_PS31

/* The default port base address for ISA clocks.
 * Some programs assume a default port for an ISA clock,
 * others do not but require a cmd line parameter.
 */
#define PCPS_DEFAULT_PORT       0x0300

/* the total number of ports acquired by a ISA clock */
#define PCPS_NUM_PORTS_ISA      4

/* the number of address lines decoded by a ISA clock */
#define PCPS_DECODE_WIDTH_ISA   10


_ext PCPS_DEV_TYPE pcps_dev_type[N_PCPS_DEV_TYPE]
#ifdef _DO_INIT
= {
  { PCPS_TYPE_PC31,      "PC31",      0,                 PCPS_REF_DCF,  PCPS_BUS_ISA },
  { PCPS_TYPE_PS31_OLD,  "PS31",      MCA_ID_PS31_OLD,   PCPS_REF_DCF,  PCPS_BUS_MCA },
  { PCPS_TYPE_PS31,      "PS31",      MCA_ID_PS31,       PCPS_REF_DCF,  PCPS_BUS_MCA },
  { PCPS_TYPE_PC32,      "PC32",      ISA_ID_PCPS,       PCPS_REF_DCF,  PCPS_BUS_ISA },
  { PCPS_TYPE_PCI32,     "PCI32",     PCI_DEV_PCI32,     PCPS_REF_DCF,  PCPS_BUS_PCI },
  { PCPS_TYPE_GPS167PC,  "GPS167PC",  0,                 PCPS_REF_GPS,  PCPS_BUS_ISA },
  { PCPS_TYPE_GPS167PCI, "GPS167PCI", PCI_DEV_GPS167PCI, PCPS_REF_GPS,  PCPS_BUS_PCI },
  { PCPS_TYPE_PCI509,    "PCI509",    PCI_DEV_PCI509,    PCPS_REF_DCF,  PCPS_BUS_PCI2 },
  { PCPS_TYPE_GPS168PCI, "GPS168PCI", PCI_DEV_GPS168PCI, PCPS_REF_GPS,  PCPS_BUS_PCI2 },
  { PCPS_TYPE_PCI510,    "PCI510",    PCI_DEV_PCI510,    PCPS_REF_DCF,  PCPS_BUS_PCI3 },
  { PCPS_TYPE_GPS169PCI, "GPS169PCI", PCI_DEV_GPS169PCI, PCPS_REF_GPS,  PCPS_BUS_PCI3 },
  { PCPS_TYPE_TCR510PCI, "TCR510PCI", PCI_DEV_TCR510PCI, PCPS_REF_IRIG, PCPS_BUS_PCI3 },
  { PCPS_TYPE_TCR167PCI, "TCR167PCI", PCI_DEV_TCR167PCI, PCPS_REF_IRIG, PCPS_BUS_PCI3 },
  { PCPS_TYPE_GPS170PCI, "GPS170PCI", PCI_DEV_GPS170PCI, PCPS_REF_GPS,  PCPS_BUS_PCI3 },
  { PCPS_TYPE_PCI511,    "PCI511",    PCI_DEV_PCI511,    PCPS_REF_DCF,  PCPS_BUS_PCI3 },
  { PCPS_TYPE_TCR511PCI, "TCR511PCI", PCI_DEV_TCR511PCI, PCPS_REF_IRIG, PCPS_BUS_PCI3 }
  // If a new device is added here, don't forget to add it also to the Windows .inf file
  // and to the Linux driver's table of supported PCI devices.
}
#endif
;


#if !defined( PCPS_MAX_DDEVS )
  #define PCPS_MAX_DDEVS   4
#endif

#if !defined( PCPS_MAX_ISA_CARDS )
  #define PCPS_MAX_ISA_CARDS  PCPS_MAX_DDEVS  // the number of ISA cards supported
#endif

_ext int pcps_isa_ports[PCPS_MAX_ISA_CARDS + 1];

_ext PCPS_DDEV pcps_ddev[PCPS_MAX_DDEVS];
_ext int n_ddevs;
_ext int curr_ddev_num;
_ext PCPS_DDEV *curr_ddev
#ifdef _DO_INIT
 = &pcps_ddev[0]
#endif
;


/* the first characters of a valid EPROM ID */

_ext const char *fw_id_ref[]
#ifdef _DO_INIT
 = {
     "PC3",      // PC31, PS31, PC32
     "PCI",      // PCI32, PCI509, PCI510, PCI511
     "GPS",      // GPS167PC, GPS167PCI, GPS168PCI, GPS169PCI, GPS170PCI
     "TCR",      // TCR510PCI, TCR167PCI, TCR511PCI
     NULL
   }
#endif
;


#define fw_id_ref_pcps    fw_id_ref[0]
#define fw_id_ref_pci32   fw_id_ref[1]
#define fw_id_ref_gps     fw_id_ref[2]



// The macros below accept a (PCPS_DDEV *) for easy access
// to the information stored in PCPS_DDEV structures.

// Access device type information:
#define _pcps_ddev_type_num( _p )        _pcps_type_num( &(_p)->dev )
#define _pcps_ddev_type_name( _p )       _pcps_type_name( &(_p)->dev )
#define _pcps_ddev_dev_id( _p )          _pcps_dev_id( &(_p)->dev )
#define _pcps_ddev_ref_type( _p )        _pcps_ref_type( &(_p)->dev )
#define _pcps_ddev_bus_flags( _p )       _pcps_bus_flags( &(_p)->dev )

// Query device type features:
#define _pcps_ddev_is_gps( _p )          _pcps_is_gps( &(_p)->dev )
#define _pcps_ddev_is_dcf( _p )          _pcps_is_dcf( &(_p)->dev )
#define _pcps_ddev_is_irig_rx( _p )      _pcps_is_irig_rx( &(_p)->dev )

#define _pcps_ddev_is_isa( _p )          _pcps_is_isa( &(_p)->dev )
#define _pcps_ddev_is_mca( _p )          _pcps_is_mca( &(_p)->dev )
#define _pcps_ddev_is_pci( _p )          _pcps_is_pci( &(_p)->dev )
#define _pcps_ddev_is_pci_s5920( _p )    _pcps_is_pci_s5920( &(_p)->dev )
#define _pcps_ddev_is_pci_asic( _p )     _pcps_is_pci_asic( &(_p)->dev )
#define _pcps_ddev_is_pci_amcc( _p )     _pcps_is_pci_amcc( &(_p)->dev )


// Access device configuration information:
#define _pcps_ddev_bus_num( _p )         _pcps_bus_num( &(_p)->dev )
#define _pcps_ddev_slot_num( _p )        _pcps_slot_num( &(_p)->dev )

#define _pcps_ddev_port_rsrc( _p, _n )   _pcps_port_rsrc( &(_p)->dev, _n )
#define _pcps_ddev_port_base( _p, _n )   _pcps_port_base( &(_p)->dev, _n )
#define _pcps_ddev_irq_num( _p )         _pcps_irq_num( &(_p)->dev )

#define _pcps_ddev_fw_rev_num( _p )      _pcps_fw_rev_num( &(_p)->dev )
#define _pcps_ddev_fw_id( _p )           _pcps_fw_id( &(_p)->dev )
#define _pcps_ddev_sernum( _p )          _pcps_sernum( &(_p)->dev )


// The macros below handle the clock device's err_flags.
#define _pcps_ddev_set_err_flags( _p, _msk ) \
        _pcps_set_err_flags( &(_p)->dev, _msk )

#define _pcps_ddev_clr_err_flags( _p, _msk ) \
        _pcps_clr_err_flags( &(_p)->dev, _msk )

#define _pcps_ddev_chk_err_flags( _p, _msk ) \
        _pcps_chk_err_flags( &(_p)->dev, _msk )


// Query whether a special feature is supported:
#define _pcps_ddev_has_feature( _p, _f ) _pcps_has_feature( &(_p)->dev, _f )
#define _pcps_ddev_can_set_time( _p )    _pcps_can_set_time( &(_p)->dev )
#define _pcps_ddev_has_serial( _p )      _pcps_has_serial( &(_p)->dev )
#define _pcps_ddev_has_sync_time( _p )   _pcps_has_sync_time( &(_p)->dev )
#define _pcps_ddev_has_ident( _p )       _pcps_has_ident( &(_p)->dev )
#define _pcps_ddev_has_utc_offs( _p )    _pcps_has_utc_offs( &(_p)->dev )
#define _pcps_ddev_has_hr_time( _p )     _pcps_has_hr_time( &(_p)->dev )
#define _pcps_ddev_has_sernum( _p )      _pcps_has_sernum( &(_p)->dev )
#define _pcps_ddev_has_cab_len( _p )     _pcps_has_cab_len( &(_p)->dev )
#define _pcps_ddev_has_tzdl( _p )        _pcps_has_tzdl( &(_p)->dev )
#define _pcps_ddev_has_pcps_tzdl( _p )   _pcps_has_pcps_tzdl( &(_p)->dev )
#define _pcps_ddev_has_tzcode( _p )      _pcps_has_tzcode( &(_p)->dev )
#define _pcps_ddev_has_tz( _p )          _pcps_has_tz( &(_p)->dev )
// The next one is supported only with a certain GPS firmware version:
#define _pcps_ddev_has_event_time( _p )  _pcps_has_event_time( &(_p)->dev )
#define _pcps_ddev_has_receiver_info( _p ) _pcps_has_receiver_info( &(_p)->dev )
#define _pcps_ddev_can_clr_ucap_buff( _p ) _pcps_can_clr_ucap_buff( &(_p)->dev )
#define _pcps_ddev_has_ucap( _p )        _pcps_has_ucap( &(_p)->dev )
#define _pcps_ddev_has_irig_tx( _p )     _pcps_has_irig_tx( &(_p)->dev )

// The macro below determines whether a DCF77 clock
// supports a higher baud rate than standard
#define _pcps_ddev_has_serial_hs( _p ) \
        _pcps_has_serial_hs( &(_p)->dev )


#define _pcps_ddev_has_signal( _p ) \
        _pcps_has_signal( &(_p)->dev )

#define _pcps_ddev_has_mod( _p ) \
        _pcps_has_mod( &(_p)->dev )


#define _pcps_ddev_has_irig( _p ) \
        _pcps_has_irig( &(_p)->dev )

#define _pcps_ddev_has_ref_offs( _p ) \
        _pcps_has_ref_offs( &(_p)->dev )

#define _pcps_ddev_has_opt_flags( _p ) \
        _pcps_has_opt_flags( &(_p)->dev )

#define _pcps_ddev_has_gps_data( _p ) \
        _pcps_has_gps_data( &(_p)->dev )

#define _pcps_ddev_has_gps_data_16( _p ) \
        _pcps_has_gps_data_16( &(_p)->dev )

#define _pcps_ddev_has_synth( _p ) \
        _pcps_has_synth( &(_p)->dev )

#define _pcps_ddev_has_generic_io( _p ) \
        _pcps_has_generic_io( &(_p)->dev )


// The macros below simplify read/write access to the clocks.

// Call the device's read function to write the command byte _cmd
// and read _n bytes to buffer _s.
#define _pcps_read( _pddev, _cmd, _s, _n )  \
  ( (_pddev)->read( &(_pddev)->dev.cfg, (_cmd), (uchar FAR *)(_s), (_n) ) )

// Write a byte _b to the radio clock device.
#define _pcps_write_byte( _pddev, _b )  \
  _pcps_read( (_pddev), (_b), NULL, 0 )


// Read data from the radio clock board to variable _s.
// The number of bytes to read is determined by the size
// of _s. The accepted type of _s depends on the _cmd code.
#define _pcps_read_var( _pddev, _cmd, _s )  \
  _pcps_read( (_pddev), (_cmd), &(_s), sizeof( (_s) ) )

// Write data from variable _s to the radio clock board .
// The number of bytes to write is determined by the size
// of _s. The accepted type of _s depends on the _cmd code.
#define _pcps_write_var( _pddev, _cmd, _s )  \
  pcps_write( (_pddev), (_cmd), (uchar FAR *)&(_s), sizeof( (_s) ) )


// Read data structures which exceed PCPS_FIFO_SIZE bytes.
// This can't be handled in a single read cycle and due to limitations
// of the clock's microprocessor these calls can up to 20 milliseconds.
// Currently these function is only used to read GPS specific data
// from GPS clocks.
#define _pcps_read_gps_var( _pddev, _cmd, _s )  \
  pcps_read_gps( (_pddev), (_cmd), (uchar FAR *) &(_s), sizeof( (_s) ) )

// The write function opposite to the read function above.
#define _pcps_write_gps_var( _pddev, _cmd, _s )  \
  pcps_write_gps( (_pddev), (_cmd), (uchar FAR *) &(_s), sizeof( (_s) ) )


// Generate a hardware reset on the radio clock board. This
// macro should be used VERY carefully and should be avoided
// if possible.
#define _pcps_force_reset( _pddev ) \
  _pcps_write_byte( (_pddev), PCPS_FORCE_RESET )


// The macros below read a radio clock's status port which
// includes the BUSY flag and the modulation signal of DCF77
// clocks. The first macro takes a (PCPS_CFG *) as argument
// whereas the second can be used with a (PCPS_DDEV *).
#define _pcps_cfg_read_status_port( _d )  ( (uchar) inp( (_d)->status_port ) )
#define _pcps_ddev_read_status_port( _d ) \
        _pcps_cfg_read_status_port( &(_d)->dev.cfg )


// The macro below checks whether a workaround is required to get/set
// IRIG cfg from a GPS169PCI with IRIG output and early firmware version
// This is handled in mbgdevio.c for direct access environments, and in
// macioctl.h for kernel device drivers.
#define _pcps_ddev_requires_irig_workaround( _d ) \
  ( ( _pcps_ddev_type_num( _d ) == PCPS_TYPE_GPS169PCI ) && \
    ( _pcps_ddev_fw_rev_num( _d ) < REV_HAS_GPS_DATA_16_GPS169PCI ) )

/* function prototypes: */

#ifdef __cplusplus
extern "C" {
#endif

/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 short pcps_write( PCPS_DDEV *pddev, uint8_t cmd, const void FAR *buffer, uint8_t count ) ;
 short pcps_generic_io( PCPS_DDEV *pddev, uint8_t type, const void FAR *in_buff, uint8_t in_cnt, void FAR *out_buff, uint8_t out_cnt ) ;
 short pcps_read_gps( PCPS_DDEV *pddev, uint8_t data_type, void FAR *buffer, uint16_t buffer_size ) ;
 short pcps_write_gps( PCPS_DDEV *pddev, uint8_t data_type, const void FAR *buffer, uint16_t buffer_size ) ;
 short pcps_get_fw_id( PCPS_DDEV *pddev, PCPS_ID_STR FAR fw_id ) ;
 short pcps_check_id( PCPS_DDEV *pddev, const char FAR *ref ) ;
 short pcps_get_rev_num( char FAR *idstr ) ;
 int pcps_read_sernum( PCPS_DDEV *pddev ) ;
 void pcps_rsrc_release( PCPS_DDEV *pddev ) ;
 void pcps_rsrc_release_all( void ) ;
 ushort pcps_port_from_pos( ushort pos ) ;
 uchar pcps_pos_from_port( ushort port ) ;
 PCPS_DEV_TYPE *pcps_get_dev_type( int bus_mask, ushort dev_id ) ;
 PCPS_DDEV *pcps_add_new_ddev( void ) ;
 void pcps_remove_last_ddev( void ) ;
 int pcps_pnp_add_device( PCPS_DDEV *pddev, int bus_flags, ushort dev_id ) ;
 int pcps_pnp_start_device( PCPS_DDEV *pddev, PCPS_BUS_NUM bus_num, PCPS_SLOT_NUM dev_fnc_num, PCPS_RSRC *prsrc ) ;
 void pcps_setup_pci_dev( PCPS_DDEV *pddev, PCPS_BUS_NUM bus_num, PCPS_SLOT_NUM dev_fnc_num ) ;
 void pcps_detect_pci_clocks( ushort vendor_id, PCPS_DEV_TYPE dev_type[], int n_dev_types ) ;
 void pcps_detect_clocks( int isa_ports[PCPS_MAX_ISA_CARDS], int isa_irqs[PCPS_MAX_ISA_CARDS] ) ;

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USE_PACK )   // set default alignment
  #pragma pack()
#endif


/* End of header body */

#undef _ext
#undef _DO_INIT

#endif  /* _PCPSDRVR_H */
