
/**************************************************************************
 *
 *  $Id: pci_qnx.h 1.3 2006/07/20 11:54:51 martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions to enable usage of Meinberg PCI function calls
 *    with QNX.
 *
 * -----------------------------------------------------------------------
 *  $Log: pci_qnx.h $
 *  Revision 1.3  2006/07/20 11:54:51  martin
 *  Account for pci_..() function calls having been renamed to _mbg_pci_..() 
 *  calls in the core library.
 *  Revision 1.2  2004/03/10 16:35:50  martin
 *  Made a (char *) cast to the last parameter of the PCI config
 *  functions as proposed by Steffen Gieseler, BitControl.
 *  Revision 1.1  2001/03/05 16:37:55  MARTIN
 *  Initial revision
 *
 **************************************************************************/

#ifndef _PCI_QNX_H
#define _PCI_QNX_H

#include <pcidefs.h>


#if defined( __QNXNTO__ )
  // Compiling for QNX Neutrino which has build-in PCI support.

  #include <hw/pci.h>

  // Before any other PCI functions can be called a program
  // must attach to a PCI server process which actually handles
  // all other requests. The pci_attach function returns an
  // (unsigned int) handle which is passed to pci_detach()
  // after all PCI requests have been made.
  #define _mbg_pci_fnc_init()      ( ( pci_handle = pci_attach( 0 ) ) == -1 )
  #define _mbg_pci_fnc_deinit()    ( pci_detach( pci_handle ) )

  #define _mbg_pci_find_bios       pci_present
  #define _mbg_pci_find_device     pci_find_device
  //    pcibios_find_device( (_did), (_vid), (_ix), (_pbn), (_pdfn) )


  #define _mbg_pci_read_cfg_byte( bus, dev_fnc, offs, addr ) \
           pci_read_config8( bus, dev_fnc, offs, 1, (char *) addr )

  #define _mbg_pci_read_cfg_word( bus, dev_fnc, offs, addr ) \
           pci_read_config16( bus, dev_fnc, offs, 1, (char *) addr )

  #define _mbg_pci_read_cfg_dword( bus, dev_fnc, offs, addr ) \
           pci_read_config32( bus, dev_fnc, offs, 1, (char *) addr )


  #define _mbg_pci_write_cfg_byte( bus, dev_fnc, offs, addr ) \
           pci_write_config8( bus, dev_fnc, offs, 1, (char *) &addr )

  #define _mbg_pci_write_cfg_word( bus, dev_fnc, offs, addr ) \
           pci_write_config16( bus, dev_fnc, offs, 1, (char *) &addr )

  #define _mbg_pci_write_cfg_dword( bus, dev_fnc, offs, addr ) \
           pci_write_config32( bus, dev_fnc, offs, 1, (char *) &addr )

#elif defined( __QNX__ )

  // compiling for QNX 4

  #include <pcidefs.h>
  #include <sys/pci.h>

  #define _mbg_pci_find_bios( _p1, _p2, _p3 )  \
          _CA_PCI_BIOS_Present( _p3, _p2, _p1 )

  #define _mbg_pci_find_device( _did, _vid, _ix, _pbn, _pdfn )  \
          _CA_PCI_Find_Device ( (_did), (_vid), (_ix), (_pbn), (_pdfn) )


  #define _mbg_pci_read_cfg_byte( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Read_Config_Byte( bus, dev_fnc, offs, 1, (char *) addr )

  #define _mbg_pci_read_cfg_word( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Read_Config_Word( bus, dev_fnc, offs, 1, (char *) addr )

  #define _mbg_pci_read_cfg_dword( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Read_Config_DWord( bus, dev_fnc, offs, 1, (char *) addr )


  #define _mbg_pci_write_cfg_byte( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Write_Config_Byte( bus, dev_fnc, offs, 1, (char *) &addr )

  #define _mbg_pci_write_cfg_word( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Write_Config_Word( bus, dev_fnc, offs, 1, (char *) &addr )

  #define _mbg_pci_write_cfg_dword( bus, dev_fnc, offs, addr ) \
          _CA_PCI_Write_Config_DWord( bus, dev_fnc, offs, 1, (char *) &addr )

#else
  // older QNX version ?
#endif

#endif  /* _PCI_QNX_H */
